@@ -7,14 +7,12 @@
 from __future__ import absolute_import, print_function
 
 import jupyter_server
-import asyncio
 import binascii
 import datetime
 import errno
 import gettext
 import hashlib
 import hmac
-import importlib
 import io
 import ipaddress
 import json
@@ -33,8 +31,8 @@ import time
 import warnings
 import webbrowser
 import urllib
+import inspect
 
-from types import ModuleType
 from base64 import encodebytes
 try:
     import resource
@@ -44,8 +42,9 @@ except ImportError:
 
 from jinja2 import Environment, FileSystemLoader
 
+from jupyter_core.paths import secure_write
 from jupyter_server.transutils import trans, _
-from jupyter_server.utils import secure_write, run_sync
+from jupyter_server.utils import run_sync
 
 # the minimum viable tornado version: needs to be kept in sync with setup.py
 MIN_TORNADO = (6, 1, 0)
@@ -108,6 +107,7 @@ from .utils import url_path_join, check_pid, url_escape, urljoin, pathname2url
 from jupyter_server.extension.serverextension import ServerExtensionApp
 from jupyter_server.extension.manager import ExtensionManager
 from jupyter_server.extension.config import ExtensionConfigManager
+from jupyter_server.traittypes import TypeFromClasses
 
 #-----------------------------------------------------------------------------
 # Module globals
@@ -517,9 +517,28 @@ flags['allow-root']=(
     _("Allow the server to be run from root user.")
 )
 flags["no-browser"] = (
-    {"ServerApp": {"open_browser": False}},
+    {
+        "ServerApp": {"open_browser": False},
+        "ExtensionApp": {"open_browser": False}
+    },
     _("Prevent the opening of the default url in the browser."),
 )
+flags["debug"] = (
+    {
+        'ServerApp': {'log_level': 'DEBUG'},
+        'ExtensionApp': {'log_level': 'DEBUG'}
+    },
+    _("Set debug level for the extension and underlying server applications.")
+)
+flags['autoreload'] = (
+    {'ServerApp': {'autoreload': True}},
+    """Autoreload the webapp
+    Enable reloading of the tornado webapp and all imported Python packages
+    when any changes are made to any Python src files in server or
+    extensions.
+    """
+)
+
 
 # Add notebook manager flags
 flags.update(boolean_flag('script', 'FileContentsManager.save_script',
@@ -635,6 +654,10 @@ class ServerApp(JupyterApp):
         help=_("Whether to allow the user to run the server as root.")
     )
 
+    autoreload = Bool(False, config=True,
+        help= ("Reload the webapp when changes are made to any Python src files.")
+    )
+
     default_url = Unicode('/', config=True,
         help=_("The default URL to redirect to from `/`")
     )
@@ -937,23 +960,20 @@ class ServerApp(JupyterApp):
     # It is sometimes important to know if + which another app (say a server extension)
     # started the serverapp to properly configure some traits.
     # This trait should not be configured by users. It will likely be set by ExtensionApp.
-    _starter_app_name = Unicode(None, allow_none=True)
+    _starter_app = Instance(JupyterApp, allow_none=True)
 
-    @validate('_starter_app_name')
+    @validate('_starter_app')
     def _validate_starter_app(self, proposal):
         # Check that a previous server extension isn't named yet
         value = proposal["value"]
-        if self._starter_app_name != None:
+        if self._starter_app != None:
             raise TraitError("Another extension was already named as the starter_server_extension.")
         return value
 
     @property
     def starter_app(self):
         """Get the Extension that started this server."""
-        name = self._starter_app_name
-        if name is None:
-            return
-        return self.extension_manager.extension_points.get(name, None).app
+        return self._starter_app
 
     open_browser = Bool(False, config=True,
                         help="""Whether to open in a browser after starting.
@@ -1116,13 +1136,43 @@ class ServerApp(JupyterApp):
         help="""If True, display controls to shut down the Jupyter server, such as menu items or buttons."""
     )
 
-    contents_manager_class = Type(
+    # REMOVE in VERSION 2.0
+    # Temporarily allow content managers to inherit from the 'notebook'
+    # package. We will deprecate this in the next major release.
+    contents_manager_class = TypeFromClasses(
         default_value=LargeFileManager,
-        klass=ContentsManager,
+        klasses=[
+            'jupyter_server.services.contents.manager.ContentsManager',
+            'notebook.services.contents.manager.ContentsManager'
+        ],
         config=True,
         help=_('The content manager class to use.')
     )
 
+    # Throws a deprecation warning to notebook based contents managers.
+    @observe('contents_manager_class')
+    def _observe_contents_manager_class(self, change):
+        new = change['new']
+        # If 'new' is a class, get a string representing the import
+        # module path.
+        if inspect.isclass(new):
+            new = new.__module__
+
+        if new.startswith('notebook'):
+            self.log.warning(
+                "The specified 'contents_manager_class' class inherits a manager from the "
+                "'notebook' package. This is not guaranteed to work in future "
+                "releases of Jupyter Server. Instead, consider switching the "
+                "manager to inherit from the 'jupyter_server' managers. "
+                "Jupyter Server will temporarily allow 'notebook' managers "
+                "until its next major release (2.x)."
+            )
+
+    @observe('notebook_dir')
+    def _update_notebook_dir(self, change):
+        self.log.warning(_("notebook_dir is deprecated, use root_dir"))
+        self.root_dir = change['new']
+
     kernel_manager_class = Type(
         default_value=MappingKernelManager,
         config=True,
@@ -1379,6 +1429,7 @@ class ServerApp(JupyterApp):
         if self.allow_origin_pat:
             self.tornado_settings['allow_origin_pat'] = re.compile(self.allow_origin_pat)
         self.tornado_settings['allow_credentials'] = self.allow_credentials
+        self.tornado_settings['autoreload'] = self.autoreload
         self.tornado_settings['cookie_options'] = self.cookie_options
         self.tornado_settings['get_secure_cookie_kwargs'] = self.get_secure_cookie_kwargs
         self.tornado_settings['token'] = self.token
